
/** TP ETI5 - CPE Lyon - 2010-2011 */


#include <v3.hpp>
#include <mesh.hpp>
#include <mesh_io_off.hpp>
#include <opengl_drawer.hpp>
#include <matrix3.hpp>
#include <navigator_tool.hpp>
#include <string_helper.hpp>
#include <glut_helper.hpp>
#include <texture.hpp>

#include <GL/glut.h>
#include <stdlib.h>
#include <iostream>
#include <cmath>
#include <string>
#include <errno.h>
#include <vector>





//draw the model
int draw_scene();
//load the model
int load_model();
//compute a random number
double get_rand();



//glut functions
int setup_callback();
static void display_callback();
static void reshape_callback (int width, int height);
static void keyboard_callback(unsigned char key,int x,int y);
static void idle_callback(int value);
static void motion_callback (int x, int y);
static void mouse_callback (int button, int action, int x, int y);



using namespace cpe;




//GLOBAL VARIABLES
//************************************//
unsigned int counter=0;
navigator_tool nav;



//test variables
cpe::mesh mesh_test;
std::vector<double> mesh_test_normal;

//load the model
int load_model()
{
    try
    {
        opengl_drawer::generate_texture(1,"data/pic.ppm");
        mesh_test=cpe::mesh::load_off_file("data/quad.off");
        mesh_test.scale_unity();
        mesh_test_normal=mesh_test.normal_vertex();
    }
    catch(std::string e)
    {
        std::cout<<"Exception: "<<e<<std::endl;
    }

    return 0;
}


//Draw the scene
int draw_scene()
{
    glEnable(GL_POLYGON_OFFSET_FILL);
    glPolygonOffset(5,5);


    glEnable(GL_LIGHTING);
    glColor3d(0.7,0.7,0.7);
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D,1);

    cpe::opengl_drawer::draw(mesh_test,mesh_test_normal);

    return 0;
}















int main(int argc, char *argv[])
{
    std::cout<<"**hello**\n"<<std::endl;
    std::cout<<"run "<<argv[0]<<" with "<<argc-1<<" parameters ... \n"<<std::endl;


  // ************* //
  // start OpenGL
  // ************* //

  //window size
  nav.screen_size_x()=800;
  nav.screen_size_y()=800;



  //init opengl & glut
  setup_opengl(argc,argv,nav);
  //setup callback functions
  setup_callback();





  // ************* //
  // RUN
  // ************* //
  print_help_start();

  //do start once
  load_model();

  //glut loop
  glutMainLoop ();


  return 0;
}



// *******************//
// Helper
// *******************//

//just a rand number \in [0,1]
double get_rand()
{
  int precision=50000;
  return double(rand()%precision)/double(precision);
}






//*******************//
//GLUT STUFF
//*******************//


//glut functions
int setup_callback()
{

    //callback
    glutDisplayFunc (display_callback);
    glutReshapeFunc (reshape_callback);
    glutKeyboardFunc(keyboard_callback);
    glutTimerFunc(33,idle_callback,1);
    glutMotionFunc (motion_callback);
    glutMouseFunc (mouse_callback);

    return 0;
}



//display
static void display_callback()
{
    display_manager_begin(nav);
    draw_scene();
    display_manager_end(nav,counter);
}


static void reshape_callback (int width, int height) {reshape_manager(width,height,nav);}
static void idle_callback(int value) {value=1;glutPostRedisplay ();glutTimerFunc( 20, idle_callback, 1);}
static void keyboard_callback(unsigned char key,int x,int y) {keyboard_manager(key,x,y,nav);}
static void motion_callback (int x, int y) {motion_manager(x,y,nav);}
static void mouse_callback (int button, int action, int x, int y) {mouse_manager(button,action,x,y,nav);}












